
{*******************************************************}
{                                                       }
{       Animated Menus                                  }
{       Utilities                                       }
{                                                       }
{       Copyright (c) 1997-2001 AnimatedMenus.com       }
{       All rights reserved.                            }
{                                                       }
{*******************************************************}


unit am2000utils;

{$I am2000.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Dialogs, Menus, Buttons, CommCtrl, ComCtrls, Registry;

const
  // messages
  wm_ShowAnimated     = wm_User + $102;
  wm_ShowSilent       = wm_User + $103;
  wm_HideAnimated     = wm_User + $104;
  wm_HideSilent       = wm_User + $105;

  // messages for popup menu form
  wm_KillAnimation    = wm_User + $101;
  wm_KillTimer        = wm_User + $106;
  wm_SetKeepSelected  = wm_User + $112; // wParam -> enable/disable keepselected
  wm_UpdateMenuBar    = wm_User + $107; // wParam <> 0 -> rebound menu bar
  wm_ActivateMenuBar  = wm_User + $108; // wParam <> 0 -> window is active
  wm_InitState        = wm_User + $113; // clears all menu states

  // messages for menu designer
  wm_SelectComponent  = wm_User + $120; // select component in Object Inspector
  wm_UpdateCaption    = wm_User + $121;
  wm_UpdateBitmap     = wm_User + $122;
  wm_SetFraming       = wm_User + $123; // wParam =1 - enable, =0 - disable 
  wm_SetDesigning     = wm_User + $124; // wParam =1 - enable, =0 - disable 
  wm_SetEnabled       = wm_User + $125; // wParam =1 - enable, =0 - disable
  wm_SetFocusItem     = wm_User + $126; // wParam = LongInt(Pointer(TMenuItem2000));


  // GetItemAt(X,Y)
  itNothing           = -1;
  itDragPane          = -2;
  itHiddenArrow       = -3;
  itTopScroll         = -4;
  itBottomScroll      = -5;

  // sizes
  iScrollButtonHeight: Integer = 9;
  iHiddenButtonHeight: Integer = 13;
  iDraggablePaneHeight: Integer = 7;
  iSystemIconWidth: Integer = 16;

  // constants for TCustomMenuBar2000.UpdateMenuBar routine
  upDoNothing         = 0;
  upRepaint           = 0;
  upForceRebound      = 1;
  upChildChanged      = 2;
  upForceRebuild      = 3;
  upRebuildToolTips   = 4;

  // vk_Menu key
  AltMask = $20000000;

  FormFlags = swp_NoMove or swp_NoSize or swp_NoActivate;
  dt_DrawTextFlags = dt_SingleLine + dt_VCenter;
  nSteps: Integer = 12;  // number of steps in menu animation
  nTimeout: Integer = 10; // cannot be more than 100
  nFirstStage: Integer = 5; // size of first step

  nShowHiddenTimeout: Integer = 2000;
  nPopupSubmenuTimeout: Integer = 250;
  nShortPopupSubmenuTimeout: Integer = 25;
  nScrollingTimeout: Integer = 100;
  nShortScrollingTimeout: Integer = 10;
  nHotTrackTimeout: Integer = 200;

  iMaxItemWidth: Integer = 300;
  iSubmenuTriangleWidth: Integer = 20;
  iToolTipWindowWidth: Integer = 300;
  iMaxToolTipShowingTime: Integer = 7000;


  // time out before menu item will be demoted in seconds
  // (6 hours is default)
  iDemotingTimeOut: DWord = 24*60*60;

  miNothingToInsert = Pointer(-1);
  miInsertAtTheEnd = Pointer(-2);

  // Animated Menus version
  AnimatedMenusVersion = 2.5;
  AnimatedMenusMajorVersion = 2;
  AnimatedMenusMinorVersion = 5;
  AnimatedMenusBuildNumber = 690;
  SVersion = 'Animated Menus 2.5 Build #690';

const
  Pattern: TBitmap  = nil;

  // Custom Sounds - set your favorite
  MenuPopupSound    : String = 'MenuPopup';      // or 'c:\laser.wav'
  MenuCloseSound    : String = '';
  MenuCommandSound  : String = 'MenuCommand';
  // set this param to True to load and play sound from resource
  MenuSoundResource : Boolean = False;

  // current menu that popped up
  ActivePopupMenu   : TPopupMenu = nil;

  // current menu item to move
  ActiveMenuItem    : TMenuItem = nil;
  InsertBeforeItem  : TMenuItem = nil;
  InsertBeforeParent: TMenuItem = nil;

type
  T_AM2000_MenuWhereToSave = (msDontSave, msRegistry, msMenuFile, msXmlFile);

const

  // registry key where to keep menu state
  MenuWhereToSave   : T_AM2000_MenuWhereToSave = msRegistry;
  MenuOrderRegistry : String = '\Software\AnimatedMenus.com\AnimatedMenus/2000\MenuOrder\$(APPNAME)\';
  MenuOrderFile     : String = '$(WINDOWS)\Application Data\AnimatedMenus.com\Animated Menus\$(APPNAME)\';
  StartingTime      : DWord = 0;
  ElapsedTime       : DWord = 0;

  // global parameters
  EnablePromotion     : Boolean = True;
  EnableDemotion      : Boolean = False;
  EnableCustomization : Boolean = True;

  // list of active menu bars
  ActiveMenuBarList   : TList = nil;

  // save cursor state
  SaveCursorState     : TCursor = 0;

  // timer
  TimeoutTimer        : TTimer = nil;

const
  // list of menus that float
  FloatingMenusList       : TList            = nil;
  // locks floating menus from repainting
  IgnoreRepaintFloating   : Boolean          = False;

  // checkmarks
  bmpCheckMark            : HBitmap          = 0;
  bmpRadioItem            : HBitmap          = 0;

  // cursors
  crDrag2                 = -999;
  crDragAccept            = -998;
  crDragReject            = -997;
  curDrag2                : HCursor          = 0;
  curDragAccept           : HCursor          = 0;
  curDragReject           : HCursor          = 0;

var
  Z: array [0..256] of Char;
  mii: TMenuItemInfo;
  NonClientMetrics: TNonClientMetrics;

type
  T_AM2000_Version = type String;
  T_AM2000_DefaultIndex = type Integer;
  T_AM2000_ImageIndex = type Integer;


{ other routines }

// menu control routines
procedure HideWindowMenu(Owner: TComponent);
function AssignedActivePopupMenu2000Form: Boolean;
procedure KillActivePopupMenu2000(KillMenuBar, B: Boolean);
procedure SetStatusBarText(HintText: String);

// hooks
procedure InstallWindowsHooks;
procedure RemoveWindowsHooks;
procedure InstallOle2GMHook(hwndActiveObject: HWnd);
procedure RemoveOle2GMHook;
function GetMessageHook(Code, wParam, lParam: Integer): Integer; stdcall;
function GetCBTHook(Code: Integer; wParam: HWND; lParam: LPARAM): LRESULT; stdcall;
function GetCallWndProcHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
function GetOle2MessageHook(Code, wParam, lParam: Integer): Integer; stdcall;

// message handling procedures
procedure CheckShowHint(MenuItem: TMenuItem; ShowFloatingHint: Boolean; Form: TForm);
procedure ProcessPaintMessages;
procedure ProcessMouseMoveMessages;
procedure NewDisabledBlt(Canvas: TCanvas; X, Y: Integer; clHigh, clShadow: TColor;
  hBmp: HBitmap);
procedure ImgDisabledBlt(Canvas: TCanvas; X, Y: Integer; Images: HImageList; Index: Integer;
  clHigh, clShadow: TColor);
procedure TransBlt(Canvas: TCanvas; X, Y, Glyph, NumGlyphs: Integer; hBmp: HBitmap);
procedure PaintMenuIcon(Owner, AMC: Forms.TForm; DC: HDC; X, Y, W: Integer);
procedure FullHideCaret;
procedure FullShowCaret;
function GetMenuFontHandle: HFont;
function GetValidName(Caption: String): String;
procedure RepaintFloatingMenus;
procedure OffsetBitmap(Bitmap: TBitmap; Left, Top, Width, Height: Integer);
function StripAmpersands(S: String): String;

// Evaluation Edition routine

// nifty things
function IsAccelEx(VK: Word; const Str: String; UseFirstLetter: Boolean): Boolean;
function IsAccel2(VK: Word; const Str: String): Boolean;
function HasSubmenu(Item: TMenuItem): Boolean;
function HasSubmenuWin32(Handle: HMenu; Index: Integer): Boolean;
function AmpTextWidth(Canvas: TCanvas; S: String): Integer;
function IsShortCutEx(var Msg: TWMKey; Items: TMenuItem; DoAction: Boolean): Boolean;
function GetNumLines(S: String): Integer;  // returns num of lines in the S
procedure ClearBitmap(Bitmap: TBitmap);

function GetMnuDsgnHandle: HWND;
procedure CopyToClipboard(S: String);
function PasteFromClipboard: String;

// saves menu state from registry
procedure LoadMenuFromRegistry(Menu: TMenu);
procedure SaveMenuToRegistry(Menu: TMenu);
procedure LoadMenuItemFromRegistry(Item: TMenuItem; LoadChildItems: Boolean);
procedure SaveMenuItemToRegistry(Item: TMenuItem; SaveChildItems: Boolean);

function IsActive(Control: TWinControl): Boolean;
function IsAvailable(Control: TControl): Boolean;
function IsContained(Handle: THandle; Control: TWinControl): Boolean;
function FindItem2000(Form: TForm; Value: Word; FindKing: TFindItemKind): TMenuItem;

function IsCustomization(ACustomizable: Boolean): Boolean;
procedure EndCustomization(AcceptMove: Boolean);
procedure SetDragCursor;
procedure RestoreCursor;

function IsGradientCaptions: Boolean;
function IsOkControl(C: TControl): Boolean;

procedure SetTimer(AInterval: Cardinal; AOnTimer: TNotifyEvent);
procedure StopTimer;
procedure RestartTimer;
function IsTimerSet(AInterval: Cardinal; AOnTimer: TNotifyEvent): Boolean;

implementation

uses
  am2000menubar, am2000popupmenu, am2000mainmenu, am2000hintwindow,
  am2000menuitem, am2000cache, am2000options;

const
  // directions for GetNextToolbarButton2000
  drLeft  = -1;
  drRight = 1;

  vk_0 = Byte('0');
  vk_Z = Byte('Z');

const
  CurCaretIndex : Integer = 0;

  HGetMessageHook         : HHook            = 0;
  HGetOle2MessageHook     : HHook            = 0;
  HGetCBTHook             : HHook            = 0;
  HGetCallWndProcHook     : HHook            = 0;
  WindowsHooksCount       : Integer          = 0;


{ Routines }

function AssignedActivePopupMenu2000Form: Boolean;
begin
  Result:= (ActivePopupMenu <> nil)
    and (ActivePopupMenu is TCustomPopupMenu2000)
    and TCustomPopupMenu2000(ActivePopupMenu).FormOnScreen;
end;


procedure HideWindowMenu(Owner: TComponent);
begin
  // hides only MDIForm's menu
  if (Owner is Forms.TForm)
  and (not (Owner.Owner is Forms.TForm))
  and (Forms.TForm(Owner).Menu <> nil)
  then begin
    Forms.TForm(Owner).Menu:= nil;
  end;
end;


procedure SetStatusBarText(HintText: String);
var
  I: Integer;
  S: String;
begin
  S:= Trim(GetLongHint(HintText));

  // remove '&#13;' symbols from status bar text
  repeat
    I:= Pos('&#', S);
    if I = 0 then System.Break;
    System.Delete(S, I, 5);
    System.Insert(' ', S, I);
  until False;

  if Assigned(ActivePopupMenu)
  and Assigned(TCustomPopupMenu2000(ActivePopupMenu).StatusBar)
  then
    with TCustomPopupMenu2000(ActivePopupMenu), StatusBar do begin
      if SimplePanel
      then SimpleText:= S
      else
        if StatusBarIndex < Panels.Count
        then Panels[StatusBarIndex].Text:= S;

      Exit;
    end;
end;

// processing mousemove and paint messages --
// a bit faster than Application.ProcessMessages
// thanks to Jordan Russell
procedure ProcessPaintMessages;
var
  Msg: TMsg;
begin
  while PeekMessage(Msg, 0, wm_Paint, wm_Paint, pm_NoRemove) do begin
    case Integer(GetMessage(Msg, 0, wm_Paint, wm_Paint)) of
      -1: Exit;
      0: begin PostQuitMessage(Msg.WParam); Exit; end;
    end;
    DispatchMessage(Msg);
  end;
end;

procedure ProcessMouseMoveMessages;
var
  Msg: TMsg;
begin
  while PeekMessage(Msg, 0, wm_MouseMove, wm_MouseMove, pm_NoRemove) do begin
    case Integer(GetMessage(Msg, 0, wm_MouseMove, wm_MouseMove)) of
      -1: Exit;
      0: begin PostQuitMessage(Msg.WParam); Exit; end;
    end;
    DispatchMessage(Msg);
  end;
end;


procedure KillActivePopupMenu2000;
var
  I: Integer;
begin
  StopTimer;

  try
    if AssignedActivePopupMenu2000Form then
      with ActivePopupMenu, TCustomPopupMenu2000(ActivePopupMenu).Form do begin
        SetStatusBarText('');

        Perform(wm_KillTimer, 0, 0);
        Perform(wm_KillAnimation, 0, 0);
        Perform(wm_HideSilent, 0, LongInt(B));
      end;

    if KillMenuBar
    and (ActiveMenubarList <> nil)
    then
     for I:= 0 to ActiveMenuBarList.Count -1 do
       with TCustomMenuBar2000(ActiveMenuBarList[I]) do
         HideActiveItem;
  except
  end;

  ActivePopupMenu:= nil;
end;


{ Hooks }

function IsOkControl(C: TControl): Boolean;
begin
  Result:=
    ((C is TCustomPopupMenu2000Form) and (TForm(C).BorderStyle = bsNone))
    or (C is TCustomMenuBar2000)
    or ((C <> nil) and IsOkControl(C.Parent));
end;

function GetMessageHook(Code, wParam, lParam: Integer): Integer; stdcall;
const
  LastForm: TCustomPopupMenu2000Form = nil;
var
  M: TMsg;
  Msg: Integer;
  C: TControl;
  P: TPoint;
  I: Integer;
  MB: TCustomMenuBar2000;

  procedure ClearMessage;
  begin
    FillChar(PMsg(lParam)^, SizeOf(TMsg), 0);
  end;

  function IsFloating: Boolean;
  var
    I, R: Integer;
  begin
    Result:= False;
    if FloatingMenusList <> nil
    then
      for I:= 0 to FloatingMenusList.Count -1
      do begin
        R:= SendMessage(TForm(FloatingMenusList[I]).Handle, wm_NCHitTest, 0,
          MakeLong(P.X, P.Y));
        if R <> htError
        then begin
          Result:= True;
          Exit;
        end;
      end;
  end;

begin
  Result:= 0;

  if (Code >= 0)
  and Assigned(Application)
  and Application.Active
  and (not IsIconic(GetActiveWindow))
  then begin
    M:= PMsg(lParam)^;
    Msg:= PMsg(lParam)^.Message;

    // check for mouse messages
    if ((Msg >= wm_LButtonDblClk) and (Msg <= wm_MButtonDblClk))
    or ((Msg >= wm_NCRButtonDblClk) and (Msg <= wm_NCMButtonDblClk))
    or (Msg = wm_LButtonDown)
    or (Msg = wm_NCLButtonDown)
    or (Msg = wm_NCRButtonDown)
    then begin
      // is it a mouse click on form's client area?
//      if (Msg > wm_MouseFirst)
//      then begin
        GetCursorPos(P);
        C:= FindDragTarget(P, True);
        if Assigned(C) and IsOkControl(C) then Exit;
//      end
//      else
//        if IsFloating then Exit;

      // if not -- kil active menu
      if AssignedActivePopupMenu2000Form
      then begin
        if TCustomPopupMenu2000(ActivePopupMenu).Form.BorderStyle <> bsNone
        then Exit;

        KillActivePopupMenu2000(True, False);
      end;

      if ActiveMenuBarList <> nil
      then
        for I:= 0 to ActiveMenuBarList.Count -1 do
          with TCustomMenuBar2000(ActiveMenuBarList[I]) do
            KillActiveItem;

      FullShowCaret;
    end;

    // cancel drag
    if (Msg = wm_LButtonUp)
    and (AssignedActivePopupMenu2000Form)
    then begin
      GetCursorPos(P);
      C:= FindDragTarget(P, True);
      if not (Assigned(C) and IsOkControl(C))
      then TCustomPopupMenu2000(ActivePopupMenu).Form.Perform(Msg, M.wParam, M.lParam);
    end;

    // another key?
    if ((Msg = wm_KeyDown) or (Msg = wm_KeyUp) or (Msg = wm_SysKeyDown) or (Msg = wm_SysKeyUp) or (Msg = wm_Char))
    then begin
      // is ActivePopupMenu contains the destination of the message?
      if (ActivePopupMenu <> nil)
      and (TCustomPopupMenu2000(ActivePopupMenu).GetTopMostForm.Perform(Msg, M.wParam, M.lParam) <> 0)
      then begin
        ClearMessage;
        Exit;
      end;

      // is ActiveMenuBarList contains the destination of the message?
      if ActiveMenubarList <> nil
      then
        for I:= 0 to ActiveMenuBarList.Count -1
        do begin
          MB:= TCustomMenuBar2000(ActiveMenuBarList[I]);
          if MB.Visible
          and IsAvailable(MB.OwnerForm)
          then
            if (MB.Perform(Msg, M.wParam, M.lParam) <> 0)
            then begin
              ClearMessage;
              Exit;
            end
            else
            if IsActive(MB.OwnerForm)
            then Break;
        end;
    end; { keyboard message }
  end; { main form is active }

  Result:= CallNextHookEx(HGetMessageHook, Code, wParam, lParam);
end;

function GetOle2MessageHook(Code, wParam, lParam: Integer): Integer; stdcall;
var
  M: TMsg;
  Msg: Integer;
  I: Integer;
  C: TControl;
  P: TPoint;
begin
  Result:= 0;

  if (Code >= 0)
  and Assigned(Application)
  and Application.Active
  and (not IsIconic(GetActiveWindow))
  then begin
    M:= PMsg(lParam)^;
    Msg:= PMsg(lParam)^.Message;

    // check for mouse messages
    if ((Msg >= wm_LButtonDblClk) and (Msg <= wm_MButtonDblClk))
    or ((Msg >= wm_NCRButtonDblClk) and (Msg <= wm_NCMButtonDblClk))
    or (Msg = wm_LButtonDown)
    or (Msg = wm_NCLButtonDown)
    or (Msg = wm_NCRButtonDown)
    then begin
      // is it a mouse click on form's client area?
      if (Msg > wm_MouseFirst)
      then begin
        GetCursorPos(P);
        C:= FindDragTarget(P, True);

        if Assigned(C)
        and IsOkControl(C)
        then Exit;
      end;

      // if not -- kil active menu
      if AssignedActivePopupMenu2000Form
      then begin
        KillActivePopupMenu2000(True, False);
      end;

      if ActiveMenuBarList <> nil
      then
        for I:= 0 to ActiveMenuBarList.Count -1 do
          with TCustomMenuBar2000(ActiveMenuBarList[I]) do
            KillActiveItem;

      FullShowCaret;
    end;
  end;

  Result:= CallNextHookEx(HGetOle2MessageHook, Code, wParam, lParam);
end;

function GetCBTHook(Code: Integer; wParam: HWND; lParam: LPARAM): LRESULT; stdcall;
  // updates menu bar on new mdi form
var
  I: Integer;
begin
  if ((Code = HCBT_MINMAX)
  or (Code = HCBT_SETFOCUS))
  and (ActiveMenuBarList <> nil)
  and (ActiveMenuBarList.Count > 0)
  then
    for I:= 0 to ActiveMenuBarList.Count -1 do
      with TCustomMenuBar2000(ActiveMenuBarList[I]) do
        if (OwnerForm <> nil)
        and (OwnerForm.FormStyle = fsMdiForm)
        then PostMessage(Handle, wm_UpdateMenuBar, upChildChanged, 0);

  Result:= CallNextHookEx(HGetCBTHook, Code, wParam, lParam);
end;

function GetCallWndProcHook(Code: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
  // activate and deactivate application and main form
var
  F: DWord;
  I: Integer;
  C: TWinControl;
begin
  if (Code = HC_ACTION) then
    with PCWPStruct(lParam)^ do
      if ((Message = WM_ACTIVATE)
      or (Message = WM_ACTIVATEAPP))
      and (ActiveMenuBarList <> nil)
      then begin
        if (Message = WM_ACTIVATE)
        then F:= wParam
        else F:= LongRec(wParam).Lo;

        for I:= 0 to ActiveMenuBarList.Count -1
        do begin
          C:= TWinControl(ActiveMenuBarList[I]);
          if IsContained(hwnd, C)
          then PostMessage(C.Handle, wm_ActivateMenuBar, F, 0);
        end;

        KillActivePopupMenu2000(True, False);
      end;

  Result:= CallNextHookEx(HGetCallWndProcHook, Code, wParam, lParam);
end;


{ Other Routines }

procedure CheckShowHint(MenuItem: TMenuItem; ShowFloatingHint: Boolean; Form: TForm);
var
  S: String;
begin
  if not Assigned(MenuItem) then Exit;

  S:= '';
  if (MenuItem is TMenuItem2000)
  then
    if (TMenuItem2000(MenuItem).Control = ctlButtonArray)
    then
      with TMenuItem2000(MenuItem), AsButtonArray do begin
        if (LastItemIndex >= 0)
        and (LastItemIndex < Hints.Count)
        then S:= Hints[LastItemIndex]
      end
    else
      S:= TMenuItem2000(MenuItem).Hint
  else
    S:= MenuItem.Hint;

  // fire Application.OnHint
  Application.Hint:= S;
  Form.Hint:= S;

  if (ActivePopupMenu <> nil)
  and (TCustomPopupMenu2000(ActivePopupMenu).StatusBar <> nil)
  then SetStatusBarText(S);
end;

// thanks to Brad Stowers for this routine
procedure PaintMenuIcon(Owner, AMC: Forms.TForm; DC: HDC; X, Y, W: Integer);
var
  IconHandle, NewIcon: HIcon;
begin
  if Assigned(AMC) and (AMC.Icon.Handle <> 0)
  then IconHandle := AMC.Icon.Handle
  else
  if Assigned(AMC) and (Owner.Icon.Handle <> 0)
  then IconHandle:= Owner.Icon.Handle
  else
  if Application.Icon.Handle <> 0
  then IconHandle:= Application.Icon.Handle
  else IconHandle:= LoadIcon(0, IDI_APPLICATION);

  NewIcon:= CopyImage(IconHandle, IMAGE_ICON, W, W, $4000);
  DrawIconEx(DC, X, Y, NewIcon, 0, 0, 0, 0, DI_NORMAL);
  DeleteObject(NewIcon);
end;


procedure TransBlt(Canvas: TCanvas; X, Y, Glyph, NumGlyphs: Integer; hBmp: HBitmap);
var
  BitmapDC: HDC;
  X1, Y1, DX, XE, Cur, Trans: Integer;
  BmpInfo: Windows.TBitmap;
  oldh: HBitmap;
begin
  BmpInfo.bmHeight:= 16;
  BmpInfo.bmWidth:= 16;
  GetObject(hBmp, SizeOf(BmpInfo), @BmpInfo);

  BitmapDC:= CreateCompatibleDC(Canvas.Handle);
  oldh:= SelectObject(BitmapDC, hBmp);
  if oldh <> 0 then begin
    Trans:= GetPixel(BitmapDC, 0, BmpInfo.bmHeight -1);

    if NumGlyphs > 1 then begin
      XE:= BmpInfo.bmWidth div NumGlyphs;
      DX:= Glyph * XE;
    end
    else begin
      DX:= 0;
      XE:= BmpInfo.bmWidth;
    end;

    for X1:= 0 to XE -1 do
      for Y1:= 0 to BmpInfo.bmHeight -1 do begin
        Cur:= GetPixel(BitmapDC, X1 + DX, Y1);
        if (Cur <> Trans) then
          SetPixel(Canvas.Handle, X + X1, Y + Y1, Cur);
      end;

    SelectObject(BitmapDC, oldh);
  end;

  DeleteDC(BitmapDC);
end;

procedure NewDisabledBlt(Canvas: TCanvas; X, Y: Integer; clHigh, clShadow: TColor;
  hBmp: HBitmap);
var
  XOffset: Integer;
  BitmapDC: HDC;
  Trans: TColor;
  BmpInfo: Windows.TBitmap;
  oldh: HBitmap;

  procedure PaintColor(Color: TColor; Offset: Integer);
  var
    X1, Y1, Cur: Integer;
  begin
    for X1:= XOffset to XOffset + BmpInfo.bmWidth -1 do
      for Y1:= 0 to BmpInfo.bmHeight -1 do begin
        Cur:= GetPixel(BitmapDC, X1, Y1);

        if (Cur <> Trans)
        and (Cur and $000000FF <= 132)
        and (Cur and $0000FF00 shr 08 <= 128)
        and (Cur and $00FF0000 shr 16 <= 132)
        then
          SetPixel(Canvas.Handle, X + X1 + Offset, Y + Y1 + Offset, Color);
      end;
  end;

begin
  if hBmp = 0 then Exit;
  XOffset:= 0;

  BmpInfo.bmHeight:= 16;
  BmpInfo.bmWidth:= 16;
  GetObject(hBmp, SizeOf(BmpInfo), @BmpInfo);

  BitmapDC:= CreateCompatibleDC(Canvas.Handle);
  oldh:= SelectObject(BitmapDC, hBmp);
  if oldh <> 0
  then begin
    Trans:= GetPixel(BitmapDC, 0, BmpInfo.bmHeight -1);
    if clShadow <> clNone
    then PaintColor(ColorToRgb(clShadow), 1);

    if clHigh <> clNone
    then PaintColor(ColorToRgb(clHigh), 0);

    SelectObject(BitmapDC, oldh);
  end;

  DeleteDC(BitmapDC);
end;

procedure ImgDisabledBlt(Canvas: TCanvas; X, Y: Integer; Images: HImageList; Index: Integer;
  clHigh, clShadow: TColor);
var
  Bmp: TBitmap;
  ImageInfo: TImageInfo;
begin
  Bmp:= TBitmap.Create;
  ImageList_GetImageInfo(Images, 0, ImageInfo);
  with ImageInfo.rcImage
  do begin
    Bmp.Width:= Right - Left;
    Bmp.Height:= Bottom - Top;
  end;

  ImageList_Draw(Images, Index, Bmp.Canvas.Handle, 0, 0, ild_Transparent);
  NewDisabledBlt(Canvas, X, Y, clHigh, clShadow, Bmp.Handle);

  Bmp.Free;
end;


procedure FullHideCaret;
  // hides the caret
begin
  HideCaret(0);
  Inc(CurCaretIndex);
end;

procedure FullShowCaret;
  // shows the caret
var
  I: Integer;
begin
  for I:= CurCaretIndex downto 1 do
    ShowCaret(0);

  CurCaretIndex:= 0;
end;

function GetMenuFontHandle: HFont;
  // retrives default menu font
begin
  if SystemParametersInfo(spi_GetNonClientMetrics, 0, @NonClientMetrics, 0)
  then Result:= CreateFontIndirect(NonClientMetrics.lfMenuFont)
  else Result:= GetStockObject(SYSTEM_FONT);
end;

function GetValidName(Caption: String): String;
  // creates valid menu item name from the given caption
var
  I: Integer;
begin
  Result:= '';
  for I:= 1 to Length(Caption) do
    if Caption[I] in ['0'..'9', 'A'..'Z', '_', 'a'..'z']
    then AppendStr(Result, Caption[I]);

  if Result = '' then Result:= 'N';
  if Result[1] in ['0'..'9'] then Result:= 'N' + Result;

end;

procedure RepaintFloatingMenus;
var
  I: Integer;
begin
  if IgnoreRepaintFloating
  or (FloatingMenusList = nil)
  then Exit;

  for I:= 0 to FloatingMenusList.Count -1
  do
    TForm(FloatingMenusList[I]).Repaint;
end;

procedure OffsetBitmap(Bitmap: TBitmap; Left, Top, Width, Height: Integer);
var
  TempBitmap: TBitmap;
begin
  TempBitmap:= TBitmap.Create;
  TempBitmap.Width:= Width;
  TempBitmap.Height:= Height;
  BitBlt(TempBitmap.Canvas.Handle, 0, 0, Width, Height,
    Bitmap.Canvas.Handle, Left, Top, Bitmap.Canvas.CopyMode);
  BitBlt(Bitmap.Canvas.Handle, 0, 0, Width, Height,
    TempBitmap.Canvas.Handle, 0, 0, Bitmap.Canvas.CopyMode);

  TempBitmap.Free;
end;


procedure InstallOle2GMHook(hwndActiveObject: HWnd);
var
  lpdwProcessId: LongInt;
  lpdwThreadId: LongInt;
begin
  if HGetOle2MessageHook <> 0
  then begin
    UnhookWindowsHookEx(HGetOle2MessageHook);
    HGetOle2MessageHook:= 0;
  end;

  lpdwThreadId:= GetWindowThreadProcessId(hwndActiveObject, @lpdwProcessId);

  if lpdwThreadId <> 0
  then
//    HGetOle2MessageHook:= SetWindowsHookEx(wh_GetMessage, @GetOle2MessageHook, lpdwProcessId, GetCurrentThreadID);
end;

procedure RemoveOle2GMHook;
begin
  if HGetOle2MessageHook <> 0
  then
    UnhookWindowsHookEx(HGetOle2MessageHook);

  HGetOle2MessageHook:= 0;
end;

procedure InstallWindowsHooks;
begin
  Inc(WindowsHooksCount);
  if WindowsHooksCount <> 1 then Exit;

  // install the computer-based training hook for mdi child form
  if (HGetCBTHook = 0)
  then HGetCBTHook:= SetWindowsHookEx(WH_CBT, @GetCBTHook, 0, GetCurrentThreadID);

  // install the call window procedure hook - for gray activated
  if (HGetCallWndProcHook = 0)
  then HGetCallWndProcHook:= SetWindowsHookEx(WH_CallWndProc, @GetCallWndProcHook, 0, GetCurrentThreadID);

  // setting the hook - many thanks to Victor Santos
  // for help in solving the problems with hook
  if HGetMessageHook = 0
  then HGetMessageHook:= SetWindowsHookEx(wh_GetMessage, @GetMessageHook, 0, GetCurrentThreadID);
end;

procedure RemoveWindowsHooks;
begin
  Dec(WindowsHooksCount);
  if WindowsHooksCount <> 0 then Exit;

  // remove the 'computer-based training' hook
  if HGetCBTHook <> 0
  then UnhookWindowsHookEx(HGetCBTHook);

  // remove the 'call window procedure' hook
  if HGetCallWndProcHook <> 0
  then UnhookWindowsHookEx(HGetCallWndProcHook);

  // remove the 'get message' hook
  if HGetMessageHook <> 0
  then UnhookWindowsHookEx(HGetMessageHook);

  HGetCBTHook:= 0;
  HGetCallWndProcHook:= 0;

  HGetMessageHook:= 0;
end;



function StripAmpersands(S: String): String;
var
  P: Integer;
begin
  Result:= '';
  P:= Pos('&', S);
  while P > 0 do begin
    if P > 1
    then AppendStr(Result, Copy(S, 1, P -1));
    Delete(S, 1, P);

    if (S <> '') and (S[1] = '&')
    then begin
      AppendStr(Result, '&');
      Delete(S, 1, 1);
    end;

    P:= Pos('&', S);
  end;

  AppendStr(Result, S);
end;

function IsAccelEx(VK: Word; const Str: String; UseFirstLetter: Boolean): Boolean;
var
  S: String;
begin
  Result:= (VK in [$30..$39,$41..$5a])
    and Forms.IsAccel(VK, Str);

  if (not Result)
  and UseFirstLetter
  and (Str <> '')
  then begin
    S:= StripAmpersands(Str);
    Result:= (S <> '') and (VK = Byte(UpCase(S[1])));
  end;
end;

function IsAccel2(VK: Word; const Str: String): Boolean;
begin
  Result:= IsAccelEx(VK, Str, Pos('&', Str) = 0);
end;

function HasSubmenu(Item: TMenuItem): Boolean;
begin           
  if Item is TMenuItem2000
  then
    Result:= (GetMenuItemCount(TMenuItem2000(Item).MenuHandle) > 0) // Win32 menu
      or (Item.Count > 0)   // Delphi menu
      or (TMenuItem2000(Item).AttachMenu <> nil) // Attached menu
  else
    Result:= (Item <> nil)
      and ((GetMenuItemCount(Item.Handle) > 0) // Win32 menu
      or (Item.Count > 0));   // Delphi menu
end;

function HasSubmenuWin32(Handle: HMenu; Index: Integer): Boolean;
begin
  if Index < 0
  then begin
    Result:= False;
    Exit;
  end;

  mii.fMask:= miim_State + miim_SubMenu;
  GetMenuItemInfo(Handle, Index, True, mii);
  Result:= mii.hSubMenu <> 0;
end;

function AmpTextWidth(Canvas: TCanvas; S: String): Integer;
  // returns text width without ampersands
begin
  Result:= Canvas.TextWidth(StripAmpersands(S));
end;

function GetMnuDsgnHandle: HWND;
begin
  Result:= FindWindow('T_AM2000_MenuDesignerDlg', 'Animated Menus Designer');
end;

procedure CopyToClipboard(S: String);
var
  L: Integer;
  hglbCopy: HGLOBAL;
  lptstrCopy: PChar;
begin
  L:= (Length(S) +1) * SizeOf(Char);
  OpenClipboard(0);
  EmptyClipboard;

  hglbCopy:= GlobalAlloc(GMEM_DDESHARE, L);

  lptstrCopy:= GlobalLock(hglbCopy);
  Move(PChar(S)^, lptstrCopy^, L);
//    lptstrCopy[cch] = (TCHAR) 0;    // null character

  GlobalUnlock(hglbCopy);

  // Place the handle on the clipboard.
  SetClipboardData(cf_Text, hglbCopy);

  CloseClipboard;
end;

function PasteFromClipboard: String;
var
  hglb: HGLOBAL;
  lptstr: PChar;
begin
  Result:= '';
  OpenClipboard(0);
  hglb:= GetClipboardData(cf_Text);
  lptstr:= GlobalLock(hglb);

  if lptstr <> nil then Result:= StrPas(lptstr);

  GlobalUnlock(hglb);
  CloseClipboard;
end;


function IsShortCutEx(var Msg: TWMKey; Items: TMenuItem; DoAction: Boolean): Boolean;
type
  TClickResult = (crDisabled, crClicked, crShortCutMoved);
var
  ShortCut: TShortCut;
  ShortCutStr: String;
  ShortCutItem: TMenuItem;

  function DoClick(Item: TMenuItem): TClickResult;
    // thanks to Borland (Inprise) for this code
  begin
    Result:= crClicked;
    if Item.Parent <> nil then Result:= DoClick(Item.Parent);
    if Result = crClicked then
      if Item.Enabled
      then begin
{$IFDEF Delphi4OrHigher}
        if DoAction then Item.InitiateAction;
{$ENDIF}
        Item.Click;
      end
      else Result:= crDisabled;
  end;

  function PosShortCut(const SC, SCList: String): Boolean;
    // is shortcut delimited with semi-colons or string limits?
  var
    P, E: Integer;
  begin
    P:= Pos(SC, SCList);
    E:= P + Length(SC);
    Result:= (P <> 0)
      and ((P = 1) or (SCList[P -1] = ';'))
      and ((E > Length(SCList)) or (SCList[E] = ';'));
  end;

  function FindItemByShortCut(Items: TMenuItem): TMenuItem;
  var
    I: Integer;
    MI: TMenuItem;
  begin
    I:= 0;
    Result:= nil;
    while (I < Items.Count) and (Result = nil)
    do begin
      MI:= Items[I];
      if (MI.ShortCut = ShortCut)
      or ((MI is TMenuItem2000)
      and (PosShortCut(ShortCutStr, TMenuItem2000(Items[I]).ShortCut)))
      then begin
        Result:= Items[I];
        Exit;
      end;

      if MI.Count > 0
      then
        Result:= FindItemByShortCut(MI);

      if (MI is TMenuItem2000)
      and (TMenuItem2000(MI).AttachMenu <> nil)
      then
        Result:= FindItemByShortCut(TMenuItem2000(MI).AttachMenu.Items);

      Inc(I);
    end;
  end;

  procedure SearchForItems(Items: TMenuItem);
  var
    ClickResult: TClickResult;
  begin
    if Items = nil then Exit;

    repeat
      ClickResult:= crDisabled;
      ShortCutItem:= FindItemByShortCut(Items);

      if ShortCutItem <> nil
      then begin
        KillActivePopupMenu2000(True, False);
        ClickResult:= DoClick(ShortCutItem);
      end;

    until ClickResult <> crShortCutMoved;

    if ShortCutItem <> nil
    then begin
      Msg.Result:= 1;
      Result:= True;
    end;
  end;

begin
  Result:= False;

  // get short cut
  ShortCut:= Msg.CharCode;
  if GetKeyState(vk_Shift) < 0 then Inc(ShortCut, scShift);
  if GetKeyState(vk_Control) < 0 then Inc(ShortCut, scCtrl);
  if Msg.KeyData and AltMask <> 0 then Inc(ShortCut, scAlt);

  // get text short cut
  ShortCutStr:= ShortCutToText(ShortCut);

  // search
  if ShortCutStr <> ''
  then SearchForItems(Items);
end;

function GetNumLines(S: String): Integer;
var
  P, PS: PChar;
begin
  Result:= 1;
  PS:= PChar(S);
  repeat
    P:= StrPos(PS, PChar(SNewLine));
    if P = nil then P:= StrPos(PS, #13);
    if P = nil then Break;
    Inc(Result);
    PS:= @P[1];
  until PS[0] = #0;
end;

procedure LoadMenuFromRegistry(Menu: TMenu);
var
  R, R1: TRegistry;
  NewMenuItems: TStringList;
  SNewMenuItemsRegKey: String;
  Options: T_AM2000_Options;
  Flag: T_AM2000_Flag;

  procedure LoadItemAndSubItems(Item: TMenuItem);
  var
    I: Integer;
    L: TList;
    NewItem: TMenuItem2000;
  begin
    if Item is TMenuItem2000
    then TMenuItem2000(Item).LoadFromRegistry(R);

    // process existing menu items
    if Item.Count > 0
    then begin
      L:= TList.Create;

      for I:= 0 to Item.Count -1
      do L.Add(Item[I]);

      for I:= 0 to L.Count -1
      do LoadItemAndSubItems(TMenuItem2000(L[I]));

      L.Free;

      // process new menu items
      if (Item.Name <> '')
      and (NewMenuItems.IndexOf(Item.Name) <> -1)
      then begin
        R1.OpenKey(SNewMenuItemsRegKey + '\'  + Item.Name, True);

        for I:= 0 to R1.ReadInteger(SCountRegName) -1
        do begin
          NewItem:= TMenuItem2000.Create(Item.Owner);
          Item.Add(NewItem);
          NewItem.LoadFromString(R1.ReadString(IntToStr(I)));
        end;
      end;
    end;
  end;

begin
  // doesn't work with dynamically-created menus
  if (Menu = nil)
  or (Menu.Name = '')
  // saving into registry is disabled
  or (MenuWhereToSave <> msRegistry)
  then Exit;

  // init registry
  R:= TRegistry.Create;

  // load options
  Options:= nil;
  if Menu is TCustomMainMenu2000
  then Options:= TCustomMainMenu2000(Menu).Options;
  if Menu is TCustomPopupMenu2000
  then Options:= TCustomPopupMenu2000(Menu).Options;

  if Options <> nil
  then begin
    R.OpenKey(MenuOrderRegistry + Menu.Name + SOptionsRegName, False);

    // animation
    if R.GetDataSize('Animation') > 0
    then Options.Animation:= StrToAnimation(R.ReadString('Animation'));

    // flags
    for Flag:= Low(T_AM2000_Flag) to High(T_AM2000_Flag)
    do begin
      if R.GetDataSize(FlagToStr[Flag]) > 0
      then
        if R.ReadBool(FlagToStr[Flag])
        then Options.Flags:= Options.Flags + [Flag]
        else Options.Flags:= Options.Flags - [Flag];
    end;
  end;

  // load menu items
  R.OpenKey(MenuOrderRegistry + Menu.Name, True);

  SNewMenuItemsRegKey:= MenuOrderRegistry + Menu.Name + SNewRegKey;
  NewMenuItems:= TStringList.Create;
  NewMenuItems.Sorted:= True;
  R1:= TRegistry.Create;
  R1.OpenKey(SNewMenuItemsRegKey, False);
  R1.GetKeyNames(NewMenuItems);

  // process menu items
  LoadItemAndSubItems(Menu.Items);

  R.Free;
  R1.Free;
  NewMenuItems.Free;
end;

procedure SaveMenuToRegistry(Menu: TMenu);
var
  R: TRegistry;
  Options: T_AM2000_Options;
  Flag: T_AM2000_Flag;
begin
  Options:= nil;
  if Menu is TCustomMainMenu2000
  then Options:= TCustomMainMenu2000(Menu).Options;
  if Menu is TCustomPopupMenu2000
  then Options:= TCustomPopupMenu2000(Menu).Options;

  // saving into registry is disabled
  if (MenuWhereToSave <> msRegistry)
  or (Options = nil)
  then Exit;

  R:= TRegistry.Create;
  R.OpenKey(MenuOrderRegistry + Menu.Name + SOptionsRegName, True);

  R.WriteString('Animation', AnimationToStr[Options.Animation]);

  for Flag:= Low(T_AM2000_Flag) to High(T_AM2000_Flag)
  do R.WriteBool(FlagToStr[Flag], Flag in Options.Flags);

  R.Free;
end;

procedure LoadMenuItemFromRegistry(Item: TMenuItem; LoadChildItems: Boolean);
begin
end;

procedure SaveMenuItemToRegistry(Item: TMenuItem; SaveChildItems: Boolean);
var
  R: TRegistry;
  Menu: TMenu;

  procedure SaveChildItemsToRegistry(Item: TMenuItem);
  var
    I: Integer;
  begin
    for I:= 0 to Item.Count -1
    do begin
      if Item[I] is TMenuItem2000
      then TMenuItem2000(Item[I]).SaveToRegistry(R);
      if Item[I].Count > 0
      then SaveChildItemsToRegistry(Item[I]);
    end;
  end;

begin
  // saving into registry disabled
  if (MenuWhereToSave <> msRegistry)
  // or other problems
  or (Item = nil)
  or ((Item.Name = '')
  and ((Item.Parent = nil) or (Item.Parent.Name = '')))
  or (not (Item is TMenuItem2000))
  then Exit;

  // get parent
  Menu:= TMenuItem2000(Item).GetMenu;

  // doesn't work with dynamically-created menus
  if (Menu = nil)
  or (Menu.Name = '')
  then Exit;

  // init registry
  R:= TRegistry.Create;

  if Item.Name = ''
  then begin
    // init registry
    R.OpenKey(MenuOrderRegistry + Menu.Name + SNewRegKey + '\' + Item.Parent.Name, True);
    if R.GetDataSize(SCountRegName) > 0
    then R.WriteInteger(SCountRegName, R.ReadInteger(SCountRegName) +1)
    else R.WriteInteger(SCountRegName, 1);
  end
  else
    R.OpenKey(MenuOrderRegistry + Menu.Name, True);

  // process menu item
  TMenuItem2000(Item).SaveToRegistry(R);

  if SaveChildItems
  then SaveChildItemsToRegistry(Item);

  R.Free;
end;

function IsAvailable(Control: TControl): Boolean;
var
  C: TControl;
begin
  C:= Control;
  while (C <> nil)
  do begin
    if (not C.Visible)
    or (csLoading in C.ComponentState)
    or (csDestroying in C.ComponentState)
    or ((C is TWinControl)
      and not IsWindowEnabled(TWinControl(C).Handle))
    then Break;

    C:= C.Parent;
  end;

  Result:= C = nil;
end;

function IsOwner(Owner, Child: TComponent): Boolean;
begin
  Result:= (Owner = Child)
    or ((Child <> nil) and IsOwner(Owner, Child.Owner));
end;

function IsActive(Control: TWinControl): Boolean;
var
  F: TWinControl;
begin
  Result:= False;
  if (Control = nil)
  or (csLoading in Control.ComponentState)
  or (csDestroying in Control.ComponentState)
  or not (IsWindowEnabled(Control.Handle))
  then Exit;

  if Screen.ActiveForm = nil
  then
    Result:= Control.Handle = GetForegroundWindow
  else begin
    F:= Screen.ActiveForm;
    while (F <> nil)
    do begin
      if F = Control
      then begin
        Result:= True;
        Exit;
      end;

      F:= TWinControl(F.Owner);
    end;

    Result:= (Control is TForm)
      and (Screen.ActiveForm = TForm(Control).ActiveMdiChild);
  end;
end;

function IsContained(Handle: THandle; Control: TWinControl): Boolean;
var
  C: TComponent;
begin
  Result:= False;
  C:= Control;
  while (C <> nil) do begin
    if (C is TWinControl)
    and (TWinControl(C).Handle = Handle)
    then begin
      Result:= True;
      Exit;
    end;
    C:= C.Owner;
  end;
end;

function FindItem2000(Form: TForm; Value: Word; FindKing: TFindItemKind): TMenuItem;
var
  I: Integer;
begin
  Result:= nil;
  for I:= 0 to Form.ComponentCount -1 do
    if (Form.Components[I] is TMenuItem)
    and (TMenuItem(Form.Components[I]).Command = Value)
    then begin
      Result:= TMenuItem(Form.Components[I]);
      Exit;
    end;
end;

procedure ShowCustomizeDialog;
begin
end;

// initialize registry
procedure OpenRegistry;
var
  I: Integer;
  R: TRegistry;
  Filename: String;
begin
  Filename:= ExtractFilename(ParamStr(0));
  if (Filename = '')
  or (Filename = 'DELPHI32.EXE')
  then MenuWhereToSave:= msDontSave;

  // don't store menu state
  if (MenuWhereToSave <> msRegistry)
  then Exit;

  R:= TRegistry.Create;
  I:= Pos('$(APPNAME)', MenuOrderRegistry);
  if I <> 0 then begin
    Delete(MenuOrderRegistry, I, 10);
    Insert(Filename, MenuOrderRegistry, I);
  end;

  R.OpenKey(MenuOrderRegistry, True);
  if R.GetDataSize('ElapsedTime') > 0
  then ElapsedTime:= R.ReadInteger('ElapsedTime');
  
  StartingTime:= GetTickCount;

  R.Free;
end;

// store information about usability
// for menu item demoting
procedure CloseRegistry;
var
  R: TRegistry;
begin
  // don't store menu state
  if (MenuWhereToSave <> msRegistry)
  then Exit;

  R:= TRegistry.Create;
  R.OpenKey(MenuOrderRegistry, True);
  R.WriteInteger('ElapsedTime', ElapsedTime + (GetTickCount - StartingTime) div 1000);
  R.Free;
end;

function IsCustomization(ACustomizable: Boolean): Boolean;
begin
  Result:=
    EnableCustomization
    and ACustomizable
    and (GetAsyncKeyState(vk_LButton) and $FFFF0000 <> 0)
    and (ActiveMenuItem <> nil);
end;

procedure EndCustomization(AcceptMove: Boolean);
var
  MI: TMenuItem;
  I: Integer;
begin
  try
    if AcceptMove
    and (ActiveMenuItem <> nil)
    and (InsertBeforeItem <> miNothingToInsert)
    then begin
      // check for inheretance
      if InsertBeforeItem <> miInsertAtTheEnd
      then begin
        MI:= InsertBeforeItem;
        while (MI <> nil)
        do begin
          if MI.Parent = ActiveMenuItem
          then raise EMenuError.Create(SCannotMoveItem);

          MI:= MI.Parent;
        end;
      end;

      // check for moving into different submenu
      if (InsertBeforeItem <> miNothingToInsert)
      and (InsertBeforeParent <> nil)
      and (ActiveMenuItem.Parent <> InsertBeforeParent)
      then begin
        if ActiveMenuItem.Parent <> nil
        then ActiveMenuItem.Parent.Remove(ActiveMenuItem);
        if InsertBeforeParent <> nil
        then InsertBeforeParent.Add(ActiveMenuItem);
      end;

      if (InsertBeforeItem <> miInsertAtTheEnd)
      and (InsertBeforeItem <> nil)
      then
        if ActiveMenuItem.MenuIndex < InsertBeforeItem.MenuIndex
        then
          ActiveMenuItem.MenuIndex:= InsertBeforeItem.MenuIndex -1
        else
         ActiveMenuItem.MenuIndex:= InsertBeforeItem.MenuIndex
      else
       ActiveMenuItem.MenuIndex:= MaxInt;

      SaveMenuItemToRegistry(ActiveMenuItem, False);
    end;

  finally
    ActiveMenuItem:= nil;
    InsertBeforeItem:= miNothingToInsert;
    InsertBeforeParent:= nil;

    // repaint
    if ActiveMenuBarList <> nil
    then
      for I:= 0 to ActiveMenuBarList.Count -1
      do
        with TCustomMenuBar2000(ActiveMenuBarList[I])
        do begin
          if aiState = aiFraming
          then aiState:= aiSunken;

          PostMessage(Handle, wm_SetEnabled, 1, 0);
          Repaint;
        end;

    if ActivePopupMenu is TCustomPopupMenu2000
    then
      with TCustomPopupMenu2000(ActivePopupMenu)
      do begin
        PostMessage(Form.Handle, wm_SetFraming, 0, 0);
        Refresh;
      end;

    RestoreCursor;

  end;
end;


function IsGradientCaptions: Boolean;
const
  spi_GetGradientCaptions = $1008;
var
  S: Bool;
begin  
  if SystemParametersInfo(spi_GetGradientCaptions, 0, @S, 0)
  then Result:= S
  else Result:= False;
end;

procedure SetDragCursor;
begin
  if Screen.Cursor <> crDrag2
  then begin
    if (Screen.Cursor <> crDragAccept)
    and (Screen.Cursor <> crDragReject)
    then SaveCursorState:= Screen.Cursor;
    
    Screen.Cursor:= crDrag2;
  end;
end;

procedure RestoreCursor;
begin
  if SaveCursorState <> 0
  then Screen.Cursor:= SaveCursorState
  else Screen.Cursor:= crDefault;

  SaveCursorState:= 0;
end;

procedure ClearBitmap(Bitmap: TBitmap);
var
  h: HBitmap;
begin
  h:= Bitmap.Handle;
  Bitmap.ReleaseHandle;
  DeleteObject(h);
  Bitmap.Width:= 0;
  Bitmap.Height:= 0;
end;

procedure SetTimer(AInterval: Cardinal; AOnTimer: TNotifyEvent);
begin
  if TimeoutTimer = nil
  then TimeoutTimer:= TTimer.Create(nil);
//  else
//  if TimeoutTimer.Enabled
//  and not IsTimerSet(AInterval, AOnTimer)
//  then raise Exception.Create('Timer is active');

  TimeoutTimer.Interval:= AInterval;
  TimeoutTimer.OnTimer:= AOnTimer;
  TimeoutTimer.Enabled:= True;
end;

procedure StopTimer;
begin
  if TimeoutTimer <> nil
  then TimeoutTimer.Enabled:= False;
end;

procedure RestartTimer;
begin
  if TimeoutTimer <> nil
  then begin
    TimeoutTimer.Enabled:= False;
    TimeoutTimer.Enabled:= True;
  end;
end;

function IsTimerSet(AInterval: Cardinal; AOnTimer: TNotifyEvent): Boolean;
begin
  Result:= (TimeoutTimer <> nil)
    and (TimeoutTimer.Enabled)
    and (TimeoutTimer.Interval = AInterval)
    and (@TimeoutTimer.OnTimer = @AOnTimer);
end;

procedure FreeFloatingMenusList;
var
  I: Integer;
begin
  for I:= 0 to FloatingMenusList.Count -1
  do TForm(FloatingMenusList[I]).Free;

  FloatingMenusList.Free;
  FloatingMenusList:= nil;
end;

initialization
  // active menu2000 list for multiforms
  FloatingMenusList:= TList.Create;

  // list of active menu bars
  ActiveMenuBarList:= TList.Create;

  // structure for quering menus
  mii.cbSize:= 44;
  NonClientMetrics.cbSize := sizeof(NonClientMetrics);

  // load bitmaps & cursors
  bmpRadioItem:= LoadBitmap(HInstance, 'AM2000_SYSTEMRADIOITEM');
  bmpCheckMark:= LoadBitmap(HInstance, 'AM2000_SYSTEMCHECKMARK');
  curDrag2:= LoadCursor(HInstance, 'AM2000_DRAG');
  curDragAccept:= LoadCursor(HInstance, 'AM2000_DRAG_ACCEPT');
  curDragReject:= LoadCursor(HInstance, 'AM2000_DRAG_REJECT');

  Screen.Cursors[crDrag2]:= curDrag2;
  Screen.Cursors[crDragAccept]:= curDragAccept;
  Screen.Cursors[crDragReject]:= curDragReject;

  // open registry
  OpenRegistry;

finalization

  // free floating list
  if FloatingMenusList <> nil
  then FreeFloatingMenusList;

  ActiveMenuBarList.Free;
  ActiveMenuBarList:= nil;

  // delete bitmaps & cursors
  Screen.Cursors[crDrag2]:= 0;
  Screen.Cursors[crDragAccept]:= 0;
  Screen.Cursors[crDragReject]:= 0;

  DeleteObject(bmpCheckMark);
  DeleteObject(bmpRadioItem);
  DeleteObject(curDrag2);
  DeleteObject(curDragAccept);
  DeleteObject(curDragReject);

  CloseRegistry;

  Pattern.Free;

  if TimeoutTimer <> nil
  then TimeoutTimer.Free;

end.
